/****************************************************************************
** Meta object code from reading C++ file 'pageoptions.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../QTfrontend/ui/page/pageoptions.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pageoptions.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OptionGroupBox_t {
    QByteArrayData data[1];
    char stringdata0[15];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OptionGroupBox_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OptionGroupBox_t qt_meta_stringdata_OptionGroupBox = {
    {
QT_MOC_LITERAL(0, 0, 14) // "OptionGroupBox"

    },
    "OptionGroupBox"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OptionGroupBox[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void OptionGroupBox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject OptionGroupBox::staticMetaObject = { {
    QMetaObject::SuperData::link<IconedGroupBox::staticMetaObject>(),
    qt_meta_stringdata_OptionGroupBox.data,
    qt_meta_data_OptionGroupBox,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OptionGroupBox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OptionGroupBox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OptionGroupBox.stringdata0))
        return static_cast<void*>(this);
    return IconedGroupBox::qt_metacast(_clname);
}

int OptionGroupBox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = IconedGroupBox::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_PageOptions_t {
    QByteArrayData data[34];
    char stringdata0[466];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PageOptions_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PageOptions_t qt_meta_stringdata_PageOptions = {
    {
QT_MOC_LITERAL(0, 0, 11), // "PageOptions"
QT_MOC_LITERAL(1, 12, 16), // "newTeamRequested"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 17), // "editTeamRequested"
QT_MOC_LITERAL(4, 48, 8), // "teamName"
QT_MOC_LITERAL(5, 57, 19), // "deleteTeamRequested"
QT_MOC_LITERAL(6, 77, 15), // "forceFullscreen"
QT_MOC_LITERAL(7, 93, 5), // "index"
QT_MOC_LITERAL(8, 99, 13), // "setFullscreen"
QT_MOC_LITERAL(9, 113, 5), // "state"
QT_MOC_LITERAL(10, 119, 13), // "setResolution"
QT_MOC_LITERAL(11, 133, 10), // "setQuality"
QT_MOC_LITERAL(12, 144, 5), // "value"
QT_MOC_LITERAL(13, 150, 11), // "trimNetNick"
QT_MOC_LITERAL(14, 162, 23), // "requestEditSelectedTeam"
QT_MOC_LITERAL(15, 186, 25), // "requestDeleteSelectedTeam"
QT_MOC_LITERAL(16, 212, 14), // "savePwdChanged"
QT_MOC_LITERAL(17, 227, 18), // "colorButtonClicked"
QT_MOC_LITERAL(18, 246, 1), // "i"
QT_MOC_LITERAL(19, 248, 23), // "onColorModelDataChanged"
QT_MOC_LITERAL(20, 272, 11), // "QModelIndex"
QT_MOC_LITERAL(21, 284, 7), // "topLeft"
QT_MOC_LITERAL(22, 292, 11), // "bottomRight"
QT_MOC_LITERAL(23, 304, 18), // "onProxyTypeChanged"
QT_MOC_LITERAL(24, 323, 14), // "changeAVFormat"
QT_MOC_LITERAL(25, 338, 16), // "changeUseGameRes"
QT_MOC_LITERAL(26, 355, 17), // "changeRecordAudio"
QT_MOC_LITERAL(27, 373, 15), // "checkForUpdates"
QT_MOC_LITERAL(28, 389, 15), // "tabIndexChanged"
QT_MOC_LITERAL(29, 405, 11), // "bindUpdated"
QT_MOC_LITERAL(30, 417, 6), // "bindID"
QT_MOC_LITERAL(31, 424, 13), // "resetAllBinds"
QT_MOC_LITERAL(32, 438, 9), // "setVolume"
QT_MOC_LITERAL(33, 448, 17) // "setDefaultOptions"

    },
    "PageOptions\0newTeamRequested\0\0"
    "editTeamRequested\0teamName\0"
    "deleteTeamRequested\0forceFullscreen\0"
    "index\0setFullscreen\0state\0setResolution\0"
    "setQuality\0value\0trimNetNick\0"
    "requestEditSelectedTeam\0"
    "requestDeleteSelectedTeam\0savePwdChanged\0"
    "colorButtonClicked\0i\0onColorModelDataChanged\0"
    "QModelIndex\0topLeft\0bottomRight\0"
    "onProxyTypeChanged\0changeAVFormat\0"
    "changeUseGameRes\0changeRecordAudio\0"
    "checkForUpdates\0tabIndexChanged\0"
    "bindUpdated\0bindID\0resetAllBinds\0"
    "setVolume\0setDefaultOptions"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PageOptions[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      23,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  129,    2, 0x06 /* Public */,
       3,    1,  130,    2, 0x06 /* Public */,
       5,    1,  133,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,  136,    2, 0x08 /* Private */,
       8,    1,  139,    2, 0x08 /* Private */,
      10,    1,  142,    2, 0x08 /* Private */,
      11,    1,  145,    2, 0x08 /* Private */,
      13,    0,  148,    2, 0x08 /* Private */,
      14,    0,  149,    2, 0x08 /* Private */,
      15,    0,  150,    2, 0x08 /* Private */,
      16,    1,  151,    2, 0x08 /* Private */,
      17,    1,  154,    2, 0x08 /* Private */,
      19,    2,  157,    2, 0x08 /* Private */,
      23,    0,  162,    2, 0x08 /* Private */,
      24,    1,  163,    2, 0x08 /* Private */,
      25,    1,  166,    2, 0x08 /* Private */,
      26,    1,  169,    2, 0x08 /* Private */,
      27,    0,  172,    2, 0x08 /* Private */,
      28,    1,  173,    2, 0x08 /* Private */,
      29,    1,  176,    2, 0x08 /* Private */,
      31,    0,  179,    2, 0x08 /* Private */,
      32,    1,  180,    2, 0x08 /* Private */,
      33,    0,  183,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void, QMetaType::QString,    4,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,   18,
    QMetaType::Void, 0x80000000 | 20, 0x80000000 | 20,   21,   22,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,   30,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,

       0        // eod
};

void PageOptions::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PageOptions *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->newTeamRequested(); break;
        case 1: _t->editTeamRequested((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->deleteTeamRequested((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->forceFullscreen((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->setFullscreen((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->setResolution((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->setQuality((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->trimNetNick(); break;
        case 8: _t->requestEditSelectedTeam(); break;
        case 9: _t->requestDeleteSelectedTeam(); break;
        case 10: _t->savePwdChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->colorButtonClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->onColorModelDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 13: _t->onProxyTypeChanged(); break;
        case 14: _t->changeAVFormat((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->changeUseGameRes((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->changeRecordAudio((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->checkForUpdates(); break;
        case 18: _t->tabIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->bindUpdated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: _t->resetAllBinds(); break;
        case 21: _t->setVolume((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 22: _t->setDefaultOptions(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PageOptions::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageOptions::newTeamRequested)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PageOptions::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageOptions::editTeamRequested)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PageOptions::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageOptions::deleteTeamRequested)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PageOptions::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractPage::staticMetaObject>(),
    qt_meta_stringdata_PageOptions.data,
    qt_meta_data_PageOptions,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PageOptions::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageOptions::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PageOptions.stringdata0))
        return static_cast<void*>(this);
    return AbstractPage::qt_metacast(_clname);
}

int PageOptions::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void PageOptions::newTeamRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void PageOptions::editTeamRequested(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void PageOptions::deleteTeamRequested(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
