const char * serverMessages[] = {
QT_TRANSLATE_NOOP("server", "Nickname is already in use"),
QT_TRANSLATE_NOOP("server", "This server only allows registered users to join."),
QT_TRANSLATE_NOOP("server", "No checker rights"),
QT_TRANSLATE_NOOP("server", "Authentication failed"),
QT_TRANSLATE_NOOP("server", "60 seconds cooldown after kick"),
QT_TRANSLATE_NOOP("server", "Kicked"),
QT_TRANSLATE_NOOP("server", "kicked"),
QT_TRANSLATE_NOOP("server", "Reconnected too fast"),
QT_TRANSLATE_NOOP("server", "Ping timeout"),
QT_TRANSLATE_NOOP("server", "heads"),
QT_TRANSLATE_NOOP("server", "tails"),
QT_TRANSLATE_NOOP("server", "This server does not support replays!"),
QT_TRANSLATE_NOOP("server", "This server no longer allows unregistered players to join."),
QT_TRANSLATE_NOOP("server", "This server now allows unregistered players to join."),
QT_TRANSLATE_NOOP("server", "/info <player>: Show info about player"),
QT_TRANSLATE_NOOP("server", "/me <message>: Chat action, e.g. '/me eats pizza' becomes '* Player eats pizza'"),
QT_TRANSLATE_NOOP("server", "/rnd: Flip a virtual coin and reply with 'heads' or 'tails'"),
QT_TRANSLATE_NOOP("server", "/rnd [A] [B] [C] [...]: Reply with a random word from the given list"),
QT_TRANSLATE_NOOP("server", "/watch <id>: Watch a demo stored on the server with the given ID"),
QT_TRANSLATE_NOOP("server", "/quit: Quit the server"),
QT_TRANSLATE_NOOP("server", "/help: Show chat command help"),
QT_TRANSLATE_NOOP("server", "/callvote [arguments]: Start a vote"),
QT_TRANSLATE_NOOP("server", "/vote <yes/no>: Vote 'yes' or 'no' for active vote"),
QT_TRANSLATE_NOOP("server", "/greeting [message]: Set or clear greeting message to be shown to players who join the room"),
QT_TRANSLATE_NOOP("server", "/delegate <player>: Surrender room control to player"),
QT_TRANSLATE_NOOP("server", "/maxteams <N>: Limit maximum number of teams to N"),
QT_TRANSLATE_NOOP("server", "/global <message>: Send global chat message which can be seen by everyone on the server"),
QT_TRANSLATE_NOOP("server", "/registered_only: Toggle 'registered only' state. If enabled, only registered players can join server"),
QT_TRANSLATE_NOOP("server", "/super_power: Activate your super power. With it you can enter any room and are protected from kicking. Expires when you leave server"),
QT_TRANSLATE_NOOP("server", "/stats: Query server stats"),
QT_TRANSLATE_NOOP("server", "/force <yes/no>: Force vote result for active vote"),
QT_TRANSLATE_NOOP("server", "/fix: Force this room to stay open when it is empty"),
QT_TRANSLATE_NOOP("server", "/unfix: Undo the /fix command"),
QT_TRANSLATE_NOOP("server", "/save <config ID> <config name>: Add current room configuration as votable choice for /callvote map"),
QT_TRANSLATE_NOOP("server", "/delete <config ID>: Delete a votable room configuration"),
QT_TRANSLATE_NOOP("server", "/saveroom <file name>: Save all votable room configurations (and the greeting) of this room into a file"),
QT_TRANSLATE_NOOP("server", "/loadroom <file name>: Load votable room configurations (and greeting) from a file"),
QT_TRANSLATE_NOOP("server", "List of lobby chat commands:"),
QT_TRANSLATE_NOOP("server", "List of room chat commands:"),
QT_TRANSLATE_NOOP("server", "Commands for server admins only:"),
QT_TRANSLATE_NOOP("server", "Warning! Chat flood protection activated"),
QT_TRANSLATE_NOOP("server", "Excess flood"),
QT_TRANSLATE_NOOP("server", "Game messages flood detected - 1"),
QT_TRANSLATE_NOOP("server", "Warning! Joins flood protection activated"),
QT_TRANSLATE_NOOP("server", "Warning! Room name change flood protection activated"),
QT_TRANSLATE_NOOP("server", "This command is only available in the lobby."),
QT_TRANSLATE_NOOP("server", "This command is only available in rooms."),
QT_TRANSLATE_NOOP("server", "Unknown command or invalid parameters. Say '/help' in chat for a list of commands."),
QT_TRANSLATE_NOOP("server", "Super power activated."),
QT_TRANSLATE_NOOP("server", "room"),
QT_TRANSLATE_NOOP("server", "lobby"),
QT_TRANSLATE_NOOP("server", "(playing)"),
QT_TRANSLATE_NOOP("server", "(spectating)"),
QT_TRANSLATE_NOOP("server", "Player is not online."),
QT_TRANSLATE_NOOP("server", "The game can't be started with less than two clans!"),
QT_TRANSLATE_NOOP("server", "Empty config entry."),
QT_TRANSLATE_NOOP("server", "Access denied."),
QT_TRANSLATE_NOOP("server", "You're not the room master!"),
QT_TRANSLATE_NOOP("server", "Corrupted hedgehogs info!"),
QT_TRANSLATE_NOOP("server", "Too many teams!"),
QT_TRANSLATE_NOOP("server", "Too many hedgehogs!"),
QT_TRANSLATE_NOOP("server", "There's already a team with same name in the list."),
QT_TRANSLATE_NOOP("server", "Joining not possible: Round is in progress."),
QT_TRANSLATE_NOOP("server", "This room currently does not allow adding new teams."),
QT_TRANSLATE_NOOP("server", "Error: The team you tried to remove does not exist."),
QT_TRANSLATE_NOOP("server", "You can't remove a team you don't own."),
QT_TRANSLATE_NOOP("server", "Illegal room name! The room name must be between 1-40 characters long, must not have a trailing or leading space and must not have any of these characters: $()*+?[]^{|}"),
QT_TRANSLATE_NOOP("server", "A room with the same name already exists."),
QT_TRANSLATE_NOOP("server", "You can't kick yourself!"),
QT_TRANSLATE_NOOP("server", "You can't kick the only other player!"),
QT_TRANSLATE_NOOP("server", "The player is not in your room."),
QT_TRANSLATE_NOOP("server", "This player is protected from being kicked."),
QT_TRANSLATE_NOOP("server", "You're not the room master or a server admin!"),
QT_TRANSLATE_NOOP("server", "You're already the room master."),
QT_TRANSLATE_NOOP("server", "/maxteams: specify number from 2 to 8"),
QT_TRANSLATE_NOOP("server", "Greeting message cleared."),
QT_TRANSLATE_NOOP("server", "Greeting message set."),
QT_TRANSLATE_NOOP("server", "Available callvote commands: hedgehogs <number>, pause, newseed, map <name>, kick <player>"),
QT_TRANSLATE_NOOP("server", "/callvote kick: This is only allowed in rooms without a room master."),
QT_TRANSLATE_NOOP("server", "/callvote kick: You need to specify a nickname."),
QT_TRANSLATE_NOOP("server", "/callvote kick: No such user!"),
QT_TRANSLATE_NOOP("server", "/callvote map: No maps available."),
QT_TRANSLATE_NOOP("server", "/callvote map: No such map!"),
QT_TRANSLATE_NOOP("server", "/callvote pause: No game in progress!"),
QT_TRANSLATE_NOOP("server", "/callvote hedgehogs: Specify number from 1 to 8."),
QT_TRANSLATE_NOOP("server", "/force: Please use 'yes' or 'no'."),
QT_TRANSLATE_NOOP("server", "/vote: Please use 'yes' or 'no'."),
QT_TRANSLATE_NOOP("server", "Illegal room name! A room name must be between 1-40 characters long, must not have a trailing or leading space and must not have any of these characters: $()*+?[]^{|}"),
QT_TRANSLATE_NOOP("server", "No such room."),
QT_TRANSLATE_NOOP("server", "Room version incompatible to your Hedgewars version!"),
QT_TRANSLATE_NOOP("server", "Access denied. This room currently doesn't allow joining."),
QT_TRANSLATE_NOOP("server", "Access denied. This room is for registered users only."),
QT_TRANSLATE_NOOP("server", "You are banned from this room."),
QT_TRANSLATE_NOOP("server", "Please confirm server restart with '/restart_server yes'."),
QT_TRANSLATE_NOOP("server", "Nickname already provided."),
QT_TRANSLATE_NOOP("server", "Illegal nickname! Nicknames must be between 1-40 characters long, must not have a trailing or leading space and must not have any of these characters: $()*+?[]^{|}"),
QT_TRANSLATE_NOOP("server", "Protocol already known."),
QT_TRANSLATE_NOOP("server", "Bad number."),
QT_TRANSLATE_NOOP("server", "There's no voting going on."),
QT_TRANSLATE_NOOP("server", "You already have voted."),
QT_TRANSLATE_NOOP("server", "Your vote has been counted."),
QT_TRANSLATE_NOOP("server", "Voting closed."),
QT_TRANSLATE_NOOP("server", "Pause toggled."),
QT_TRANSLATE_NOOP("server", "New voting started"),
QT_TRANSLATE_NOOP("server", "Voting expired."),
QT_TRANSLATE_NOOP("server", "kick"),
QT_TRANSLATE_NOOP("server", "map"),
QT_TRANSLATE_NOOP("server", "pause"),
QT_TRANSLATE_NOOP("server", "new seed"),
QT_TRANSLATE_NOOP("server", "hedgehogs per team: "),
QT_TRANSLATE_NOOP("server", "You're the new room master!")
};
