/****************************************************************************
** Meta object code from reading C++ file 'pageadmin.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../QTfrontend/ui/page/pageadmin.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pageadmin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PageAdmin_t {
    QByteArrayData data[26];
    char stringdata0[285];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PageAdmin_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PageAdmin_t qt_meta_stringdata_PageAdmin = {
    {
QT_MOC_LITERAL(0, 0, 9), // "PageAdmin"
QT_MOC_LITERAL(1, 10, 19), // "setServerMessageNew"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 3), // "str"
QT_MOC_LITERAL(4, 35, 19), // "setServerMessageOld"
QT_MOC_LITERAL(5, 55, 11), // "setProtocol"
QT_MOC_LITERAL(6, 67, 5), // "proto"
QT_MOC_LITERAL(7, 73, 13), // "askServerVars"
QT_MOC_LITERAL(8, 87, 18), // "clearAccountsCache"
QT_MOC_LITERAL(9, 106, 15), // "bansListRequest"
QT_MOC_LITERAL(10, 122, 9), // "removeBan"
QT_MOC_LITERAL(11, 132, 5), // "banIP"
QT_MOC_LITERAL(12, 138, 2), // "ip"
QT_MOC_LITERAL(13, 141, 6), // "reason"
QT_MOC_LITERAL(14, 148, 7), // "seconds"
QT_MOC_LITERAL(15, 156, 7), // "banNick"
QT_MOC_LITERAL(16, 164, 4), // "nick"
QT_MOC_LITERAL(17, 169, 16), // "serverMessageNew"
QT_MOC_LITERAL(18, 186, 16), // "serverMessageOld"
QT_MOC_LITERAL(19, 203, 8), // "protocol"
QT_MOC_LITERAL(20, 212, 11), // "setBansList"
QT_MOC_LITERAL(21, 224, 4), // "bans"
QT_MOC_LITERAL(22, 229, 9), // "smChanged"
QT_MOC_LITERAL(23, 239, 12), // "onAddClicked"
QT_MOC_LITERAL(24, 252, 15), // "onRemoveClicked"
QT_MOC_LITERAL(25, 268, 16) // "onRefreshClicked"

    },
    "PageAdmin\0setServerMessageNew\0\0str\0"
    "setServerMessageOld\0setProtocol\0proto\0"
    "askServerVars\0clearAccountsCache\0"
    "bansListRequest\0removeBan\0banIP\0ip\0"
    "reason\0seconds\0banNick\0nick\0"
    "serverMessageNew\0serverMessageOld\0"
    "protocol\0setBansList\0bans\0smChanged\0"
    "onAddClicked\0onRemoveClicked\0"
    "onRefreshClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PageAdmin[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   99,    2, 0x06 /* Public */,
       4,    1,  102,    2, 0x06 /* Public */,
       5,    1,  105,    2, 0x06 /* Public */,
       7,    0,  108,    2, 0x06 /* Public */,
       8,    0,  109,    2, 0x06 /* Public */,
       9,    0,  110,    2, 0x06 /* Public */,
      10,    1,  111,    2, 0x06 /* Public */,
      11,    3,  114,    2, 0x06 /* Public */,
      15,    3,  121,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      17,    1,  128,    2, 0x0a /* Public */,
      18,    1,  131,    2, 0x0a /* Public */,
      19,    1,  134,    2, 0x0a /* Public */,
      20,    1,  137,    2, 0x0a /* Public */,
      22,    0,  140,    2, 0x08 /* Private */,
      23,    0,  141,    2, 0x08 /* Private */,
      24,    0,  142,    2, 0x08 /* Private */,
      25,    0,  143,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Int,   12,   13,   14,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Int,   16,   13,   14,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, QMetaType::QStringList,   21,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PageAdmin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PageAdmin *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setServerMessageNew((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->setServerMessageOld((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->setProtocol((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->askServerVars(); break;
        case 4: _t->clearAccountsCache(); break;
        case 5: _t->bansListRequest(); break;
        case 6: _t->removeBan((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->banIP((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 8: _t->banNick((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 9: _t->serverMessageNew((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->serverMessageOld((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->protocol((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->setBansList((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 13: _t->smChanged(); break;
        case 14: _t->onAddClicked(); break;
        case 15: _t->onRemoveClicked(); break;
        case 16: _t->onRefreshClicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PageAdmin::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageAdmin::setServerMessageNew)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PageAdmin::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageAdmin::setServerMessageOld)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PageAdmin::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageAdmin::setProtocol)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (PageAdmin::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageAdmin::askServerVars)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (PageAdmin::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageAdmin::clearAccountsCache)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (PageAdmin::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageAdmin::bansListRequest)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (PageAdmin::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageAdmin::removeBan)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (PageAdmin::*)(const QString & , const QString & , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageAdmin::banIP)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (PageAdmin::*)(const QString & , const QString & , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageAdmin::banNick)) {
                *result = 8;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PageAdmin::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractPage::staticMetaObject>(),
    qt_meta_stringdata_PageAdmin.data,
    qt_meta_data_PageAdmin,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PageAdmin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageAdmin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PageAdmin.stringdata0))
        return static_cast<void*>(this);
    return AbstractPage::qt_metacast(_clname);
}

int PageAdmin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void PageAdmin::setServerMessageNew(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PageAdmin::setServerMessageOld(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void PageAdmin::setProtocol(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void PageAdmin::askServerVars()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void PageAdmin::clearAccountsCache()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void PageAdmin::bansListRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void PageAdmin::removeBan(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void PageAdmin::banIP(const QString & _t1, const QString & _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void PageAdmin::banNick(const QString & _t1, const QString & _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
