/****************************************************************************
** Meta object code from reading C++ file 'pagenetgame.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../QTfrontend/ui/page/pagenetgame.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pagenetgame.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PageNetGame_t {
    QByteArrayData data[14];
    char stringdata0[162];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PageNetGame_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PageNetGame_t qt_meta_stringdata_PageNetGame = {
    {
QT_MOC_LITERAL(0, 0, 11), // "PageNetGame"
QT_MOC_LITERAL(1, 12, 12), // "SetupClicked"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 20), // "askForUpdateRoomName"
QT_MOC_LITERAL(4, 47, 11), // "setRoomName"
QT_MOC_LITERAL(5, 59, 8), // "roomName"
QT_MOC_LITERAL(6, 68, 14), // "setReadyStatus"
QT_MOC_LITERAL(7, 83, 7), // "isReady"
QT_MOC_LITERAL(8, 91, 7), // "setUser"
QT_MOC_LITERAL(9, 99, 8), // "nickname"
QT_MOC_LITERAL(10, 108, 13), // "onUpdateClick"
QT_MOC_LITERAL(11, 122, 13), // "setMasterMode"
QT_MOC_LITERAL(12, 136, 8), // "isMaster"
QT_MOC_LITERAL(13, 145, 16) // "onRoomNameEdited"

    },
    "PageNetGame\0SetupClicked\0\0"
    "askForUpdateRoomName\0setRoomName\0"
    "roomName\0setReadyStatus\0isReady\0setUser\0"
    "nickname\0onUpdateClick\0setMasterMode\0"
    "isMaster\0onRoomNameEdited"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PageNetGame[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   54,    2, 0x06 /* Public */,
       3,    1,   55,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   58,    2, 0x0a /* Public */,
       6,    1,   61,    2, 0x0a /* Public */,
       8,    1,   64,    2, 0x0a /* Public */,
      10,    0,   67,    2, 0x0a /* Public */,
      11,    1,   68,    2, 0x0a /* Public */,
      13,    0,   71,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void,

       0        // eod
};

void PageNetGame::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PageNetGame *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->SetupClicked(); break;
        case 1: _t->askForUpdateRoomName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->setRoomName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->setReadyStatus((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->setUser((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->onUpdateClick(); break;
        case 6: _t->setMasterMode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->onRoomNameEdited(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PageNetGame::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageNetGame::SetupClicked)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PageNetGame::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageNetGame::askForUpdateRoomName)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PageNetGame::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractPage::staticMetaObject>(),
    qt_meta_stringdata_PageNetGame.data,
    qt_meta_data_PageNetGame,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PageNetGame::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageNetGame::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PageNetGame.stringdata0))
        return static_cast<void*>(this);
    return AbstractPage::qt_metacast(_clname);
}

int PageNetGame::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void PageNetGame::SetupClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void PageNetGame::askForUpdateRoomName(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
