/****************************************************************************
** Meta object code from reading C++ file 'pagegamestats.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../QTfrontend/ui/page/pagegamestats.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pagegamestats.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FitGraphicsView_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FitGraphicsView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FitGraphicsView_t qt_meta_stringdata_FitGraphicsView = {
    {
QT_MOC_LITERAL(0, 0, 15) // "FitGraphicsView"

    },
    "FitGraphicsView"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FitGraphicsView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void FitGraphicsView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FitGraphicsView::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsView::staticMetaObject>(),
    qt_meta_stringdata_FitGraphicsView.data,
    qt_meta_data_FitGraphicsView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FitGraphicsView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FitGraphicsView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FitGraphicsView.stringdata0))
        return static_cast<void*>(this);
    return QGraphicsView::qt_metacast(_clname);
}

int FitGraphicsView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsView::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_PageGameStats_t {
    QByteArrayData data[13];
    char stringdata0[145];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PageGameStats_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PageGameStats_t qt_meta_stringdata_PageGameStats = {
    {
QT_MOC_LITERAL(0, 0, 13), // "PageGameStats"
QT_MOC_LITERAL(1, 14, 17), // "saveDemoRequested"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 20), // "restartGameRequested"
QT_MOC_LITERAL(4, 54, 9), // "GameStats"
QT_MOC_LITERAL(5, 64, 4), // "type"
QT_MOC_LITERAL(6, 69, 4), // "info"
QT_MOC_LITERAL(7, 74, 5), // "clear"
QT_MOC_LITERAL(8, 80, 11), // "renderStats"
QT_MOC_LITERAL(9, 92, 17), // "restartBtnVisible"
QT_MOC_LITERAL(10, 110, 7), // "visible"
QT_MOC_LITERAL(11, 118, 18), // "saveDemoBtnEnabled"
QT_MOC_LITERAL(12, 137, 7) // "enabled"

    },
    "PageGameStats\0saveDemoRequested\0\0"
    "restartGameRequested\0GameStats\0type\0"
    "info\0clear\0renderStats\0restartBtnVisible\0"
    "visible\0saveDemoBtnEnabled\0enabled"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PageGameStats[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    0,   50,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    2,   51,    2, 0x0a /* Public */,
       7,    0,   56,    2, 0x0a /* Public */,
       8,    0,   57,    2, 0x0a /* Public */,
       9,    1,   58,    2, 0x0a /* Public */,
      11,    1,   61,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Char, QMetaType::QString,    5,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   12,

       0        // eod
};

void PageGameStats::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PageGameStats *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->saveDemoRequested(); break;
        case 1: _t->restartGameRequested(); break;
        case 2: _t->GameStats((*reinterpret_cast< char(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 3: _t->clear(); break;
        case 4: _t->renderStats(); break;
        case 5: _t->restartBtnVisible((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->saveDemoBtnEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PageGameStats::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageGameStats::saveDemoRequested)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PageGameStats::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageGameStats::restartGameRequested)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PageGameStats::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractPage::staticMetaObject>(),
    qt_meta_stringdata_PageGameStats.data,
    qt_meta_data_PageGameStats,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PageGameStats::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageGameStats::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PageGameStats.stringdata0))
        return static_cast<void*>(this);
    return AbstractPage::qt_metacast(_clname);
}

int PageGameStats::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void PageGameStats::saveDemoRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void PageGameStats::restartGameRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
