/****************************************************************************
** Meta object code from reading C++ file 'game.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../QTfrontend/game.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'game.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_HWGame_t {
    QByteArrayData data[26];
    char stringdata0[285];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_HWGame_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_HWGame_t qt_meta_stringdata_HWGame = {
    {
QT_MOC_LITERAL(0, 0, 6), // "HWGame"
QT_MOC_LITERAL(1, 7, 7), // "SendNet"
QT_MOC_LITERAL(2, 15, 0), // ""
QT_MOC_LITERAL(3, 16, 3), // "msg"
QT_MOC_LITERAL(4, 20, 8), // "SendChat"
QT_MOC_LITERAL(5, 29, 15), // "SendTeamMessage"
QT_MOC_LITERAL(6, 45, 16), // "GameStateChanged"
QT_MOC_LITERAL(7, 62, 9), // "GameState"
QT_MOC_LITERAL(8, 72, 9), // "gameState"
QT_MOC_LITERAL(9, 82, 19), // "DemoPresenceChanged"
QT_MOC_LITERAL(10, 102, 7), // "hasDemo"
QT_MOC_LITERAL(11, 110, 9), // "GameStats"
QT_MOC_LITERAL(12, 120, 4), // "type"
QT_MOC_LITERAL(13, 125, 4), // "info"
QT_MOC_LITERAL(14, 130, 10), // "HaveRecord"
QT_MOC_LITERAL(15, 141, 10), // "RecordType"
QT_MOC_LITERAL(16, 152, 6), // "record"
QT_MOC_LITERAL(17, 159, 12), // "ErrorMessage"
QT_MOC_LITERAL(18, 172, 16), // "CampStateChanged"
QT_MOC_LITERAL(19, 189, 20), // "TrainingStateChanged"
QT_MOC_LITERAL(20, 210, 18), // "SendConsoleCommand"
QT_MOC_LITERAL(21, 229, 7), // "command"
QT_MOC_LITERAL(22, 237, 7), // "FromNet"
QT_MOC_LITERAL(23, 245, 11), // "FromNetChat"
QT_MOC_LITERAL(24, 257, 14), // "FromNetWarning"
QT_MOC_LITERAL(25, 272, 12) // "FromNetError"

    },
    "HWGame\0SendNet\0\0msg\0SendChat\0"
    "SendTeamMessage\0GameStateChanged\0"
    "GameState\0gameState\0DemoPresenceChanged\0"
    "hasDemo\0GameStats\0type\0info\0HaveRecord\0"
    "RecordType\0record\0ErrorMessage\0"
    "CampStateChanged\0TrainingStateChanged\0"
    "SendConsoleCommand\0command\0FromNet\0"
    "FromNetChat\0FromNetWarning\0FromNetError"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_HWGame[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      11,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   89,    2, 0x06 /* Public */,
       4,    1,   92,    2, 0x06 /* Public */,
       5,    1,   95,    2, 0x06 /* Public */,
       6,    1,   98,    2, 0x06 /* Public */,
       9,    1,  101,    2, 0x06 /* Public */,
      11,    2,  104,    2, 0x06 /* Public */,
      14,    2,  109,    2, 0x06 /* Public */,
      17,    1,  114,    2, 0x06 /* Public */,
      18,    1,  117,    2, 0x06 /* Public */,
      19,    1,  120,    2, 0x06 /* Public */,
      20,    1,  123,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      22,    1,  126,    2, 0x0a /* Public */,
      23,    1,  129,    2, 0x0a /* Public */,
      24,    1,  132,    2, 0x0a /* Public */,
      25,    1,  135,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QByteArray,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Char, QMetaType::QString,   12,   13,
    QMetaType::Void, 0x80000000 | 15, QMetaType::QByteArray,   12,   16,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,   21,

 // slots: parameters
    QMetaType::Void, QMetaType::QByteArray,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,

       0        // eod
};

void HWGame::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HWGame *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->SendNet((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 1: _t->SendChat((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->SendTeamMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->GameStateChanged((*reinterpret_cast< GameState(*)>(_a[1]))); break;
        case 4: _t->DemoPresenceChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->GameStats((*reinterpret_cast< char(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 6: _t->HaveRecord((*reinterpret_cast< RecordType(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        case 7: _t->ErrorMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->CampStateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->TrainingStateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->SendConsoleCommand((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->FromNet((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 12: _t->FromNetChat((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->FromNetWarning((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: _t->FromNetError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HWGame::*)(const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWGame::SendNet)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (HWGame::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWGame::SendChat)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (HWGame::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWGame::SendTeamMessage)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (HWGame::*)(GameState );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWGame::GameStateChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (HWGame::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWGame::DemoPresenceChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (HWGame::*)(char , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWGame::GameStats)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (HWGame::*)(RecordType , const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWGame::HaveRecord)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (HWGame::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWGame::ErrorMessage)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (HWGame::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWGame::CampStateChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (HWGame::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWGame::TrainingStateChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (HWGame::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWGame::SendConsoleCommand)) {
                *result = 10;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject HWGame::staticMetaObject = { {
    QMetaObject::SuperData::link<TCPBase::staticMetaObject>(),
    qt_meta_stringdata_HWGame.data,
    qt_meta_data_HWGame,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *HWGame::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HWGame::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_HWGame.stringdata0))
        return static_cast<void*>(this);
    return TCPBase::qt_metacast(_clname);
}

int HWGame::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TCPBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void HWGame::SendNet(const QByteArray & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void HWGame::SendChat(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void HWGame::SendTeamMessage(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void HWGame::GameStateChanged(GameState _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void HWGame::DemoPresenceChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void HWGame::GameStats(char _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void HWGame::HaveRecord(RecordType _t1, const QByteArray & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void HWGame::ErrorMessage(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void HWGame::CampStateChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void HWGame::TrainingStateChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void HWGame::SendConsoleCommand(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
