/****************************************************************************
** Meta object code from reading C++ file 'mapContainer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../QTfrontend/ui/widget/mapContainer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mapContainer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_HWMapContainer_t {
    QByteArrayData data[63];
    char stringdata0[736];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_HWMapContainer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_HWMapContainer_t qt_meta_stringdata_HWMapContainer = {
    {
QT_MOC_LITERAL(0, 0, 14), // "HWMapContainer"
QT_MOC_LITERAL(1, 15, 11), // "seedChanged"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 4), // "seed"
QT_MOC_LITERAL(4, 33, 10), // "mapChanged"
QT_MOC_LITERAL(5, 44, 3), // "map"
QT_MOC_LITERAL(6, 48, 12), // "themeChanged"
QT_MOC_LITERAL(7, 61, 5), // "theme"
QT_MOC_LITERAL(8, 67, 17), // "newTemplateFilter"
QT_MOC_LITERAL(9, 85, 6), // "filter"
QT_MOC_LITERAL(10, 92, 13), // "mapgenChanged"
QT_MOC_LITERAL(11, 106, 12), // "MapGenerator"
QT_MOC_LITERAL(12, 119, 1), // "m"
QT_MOC_LITERAL(13, 121, 15), // "mazeSizeChanged"
QT_MOC_LITERAL(14, 137, 1), // "s"
QT_MOC_LITERAL(15, 139, 21), // "mapFeatureSizeChanged"
QT_MOC_LITERAL(16, 161, 16), // "drawMapRequested"
QT_MOC_LITERAL(17, 178, 15), // "drawnMapChanged"
QT_MOC_LITERAL(18, 194, 4), // "data"
QT_MOC_LITERAL(19, 199, 22), // "askForGeneratedPreview"
QT_MOC_LITERAL(20, 222, 7), // "setSeed"
QT_MOC_LITERAL(21, 230, 9), // "setScript"
QT_MOC_LITERAL(22, 240, 6), // "script"
QT_MOC_LITERAL(23, 247, 11), // "scriptparam"
QT_MOC_LITERAL(24, 259, 6), // "setMap"
QT_MOC_LITERAL(25, 266, 8), // "setTheme"
QT_MOC_LITERAL(26, 275, 17), // "setTemplateFilter"
QT_MOC_LITERAL(27, 293, 9), // "setMapgen"
QT_MOC_LITERAL(28, 303, 11), // "setMazeSize"
QT_MOC_LITERAL(29, 315, 4), // "size"
QT_MOC_LITERAL(30, 320, 14), // "setFeatureSize"
QT_MOC_LITERAL(31, 335, 15), // "setDrawnMapData"
QT_MOC_LITERAL(32, 351, 2), // "ar"
QT_MOC_LITERAL(33, 354, 19), // "setAllMapParameters"
QT_MOC_LITERAL(34, 374, 8), // "mazesize"
QT_MOC_LITERAL(35, 383, 4), // "tmpl"
QT_MOC_LITERAL(36, 388, 11), // "featureSize"
QT_MOC_LITERAL(37, 400, 16), // "updateModelViews"
QT_MOC_LITERAL(38, 417, 21), // "onPreviewMapDestroyed"
QT_MOC_LITERAL(39, 439, 9), // "setMaster"
QT_MOC_LITERAL(40, 449, 6), // "master"
QT_MOC_LITERAL(41, 456, 19), // "setRandomWithoutDLC"
QT_MOC_LITERAL(42, 476, 10), // "withoutDLC"
QT_MOC_LITERAL(43, 487, 15), // "onImageReceived"
QT_MOC_LITERAL(44, 503, 8), // "newImage"
QT_MOC_LITERAL(45, 512, 10), // "setHHLimit"
QT_MOC_LITERAL(46, 523, 7), // "hhLimit"
QT_MOC_LITERAL(47, 531, 13), // "setRandomSeed"
QT_MOC_LITERAL(48, 545, 14), // "setRandomTheme"
QT_MOC_LITERAL(49, 560, 12), // "setRandomMap"
QT_MOC_LITERAL(50, 573, 10), // "setNewSeed"
QT_MOC_LITERAL(51, 584, 7), // "newSeed"
QT_MOC_LITERAL(52, 592, 14), // "mapTypeChanged"
QT_MOC_LITERAL(53, 607, 15), // "showThemePrompt"
QT_MOC_LITERAL(54, 623, 11), // "updateTheme"
QT_MOC_LITERAL(55, 635, 11), // "QModelIndex"
QT_MOC_LITERAL(56, 647, 7), // "current"
QT_MOC_LITERAL(57, 655, 16), // "staticMapChanged"
QT_MOC_LITERAL(58, 672, 3), // "old"
QT_MOC_LITERAL(59, 676, 17), // "missionMapChanged"
QT_MOC_LITERAL(60, 694, 11), // "loadDrawing"
QT_MOC_LITERAL(61, 706, 14), // "showSeedPrompt"
QT_MOC_LITERAL(62, 721, 14) // "previewClicked"

    },
    "HWMapContainer\0seedChanged\0\0seed\0"
    "mapChanged\0map\0themeChanged\0theme\0"
    "newTemplateFilter\0filter\0mapgenChanged\0"
    "MapGenerator\0m\0mazeSizeChanged\0s\0"
    "mapFeatureSizeChanged\0drawMapRequested\0"
    "drawnMapChanged\0data\0askForGeneratedPreview\0"
    "setSeed\0setScript\0script\0scriptparam\0"
    "setMap\0setTheme\0setTemplateFilter\0"
    "setMapgen\0setMazeSize\0size\0setFeatureSize\0"
    "setDrawnMapData\0ar\0setAllMapParameters\0"
    "mazesize\0tmpl\0featureSize\0updateModelViews\0"
    "onPreviewMapDestroyed\0setMaster\0master\0"
    "setRandomWithoutDLC\0withoutDLC\0"
    "onImageReceived\0newImage\0setHHLimit\0"
    "hhLimit\0setRandomSeed\0setRandomTheme\0"
    "setRandomMap\0setNewSeed\0newSeed\0"
    "mapTypeChanged\0showThemePrompt\0"
    "updateTheme\0QModelIndex\0current\0"
    "staticMapChanged\0old\0missionMapChanged\0"
    "loadDrawing\0showSeedPrompt\0previewClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_HWMapContainer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      40,   14, // methods
       1,  330, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  214,    2, 0x06 /* Public */,
       4,    1,  217,    2, 0x06 /* Public */,
       6,    1,  220,    2, 0x06 /* Public */,
       8,    1,  223,    2, 0x06 /* Public */,
      10,    1,  226,    2, 0x06 /* Public */,
      13,    1,  229,    2, 0x06 /* Public */,
      15,    1,  232,    2, 0x06 /* Public */,
      16,    0,  235,    2, 0x06 /* Public */,
      17,    1,  236,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      19,    0,  239,    2, 0x0a /* Public */,
      20,    1,  240,    2, 0x0a /* Public */,
      21,    2,  243,    2, 0x0a /* Public */,
      24,    1,  248,    2, 0x0a /* Public */,
      25,    1,  251,    2, 0x0a /* Public */,
      26,    1,  254,    2, 0x0a /* Public */,
      27,    1,  257,    2, 0x0a /* Public */,
      28,    1,  260,    2, 0x0a /* Public */,
      30,    1,  263,    2, 0x0a /* Public */,
      31,    1,  266,    2, 0x0a /* Public */,
      33,    6,  269,    2, 0x0a /* Public */,
      37,    0,  282,    2, 0x0a /* Public */,
      38,    1,  283,    2, 0x0a /* Public */,
      39,    1,  286,    2, 0x0a /* Public */,
      41,    1,  289,    2, 0x0a /* Public */,
      43,    1,  292,    2, 0x08 /* Private */,
      45,    1,  295,    2, 0x08 /* Private */,
      47,    0,  298,    2, 0x08 /* Private */,
      48,    0,  299,    2, 0x08 /* Private */,
      49,    0,  300,    2, 0x08 /* Private */,
      50,    1,  301,    2, 0x08 /* Private */,
      52,    1,  304,    2, 0x08 /* Private */,
      53,    0,  307,    2, 0x08 /* Private */,
      54,    1,  308,    2, 0x08 /* Private */,
      57,    2,  311,    2, 0x08 /* Private */,
      57,    1,  316,    2, 0x28 /* Private | MethodCloned */,
      59,    2,  319,    2, 0x08 /* Private */,
      59,    1,  324,    2, 0x28 /* Private | MethodCloned */,
      60,    0,  327,    2, 0x08 /* Private */,
      61,    0,  328,    2, 0x08 /* Private */,
      62,    0,  329,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QByteArray,   18,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   22,   23,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, QMetaType::Int,   29,
    QMetaType::Void, QMetaType::Int,   29,
    QMetaType::Void, QMetaType::QByteArray,   32,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 11, QMetaType::Int, QMetaType::QString, QMetaType::Int, QMetaType::Int,    5,   12,   34,    3,   35,   36,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QObjectStar,    5,
    QMetaType::Void, QMetaType::Bool,   40,
    QMetaType::Void, QMetaType::Bool,   42,
    QMetaType::Void, QMetaType::QPixmap,   44,
    QMetaType::Void, QMetaType::Int,   46,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   51,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 55,   56,
    QMetaType::Void, 0x80000000 | 55, 0x80000000 | 55,    5,   58,
    QMetaType::Void, 0x80000000 | 55,    5,
    QMetaType::Void, 0x80000000 | 55, 0x80000000 | 55,    5,   58,
    QMetaType::Void, 0x80000000 | 55,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      40, QMetaType::Bool, 0x00095103,

       0        // eod
};

void HWMapContainer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HWMapContainer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->seedChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->mapChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->themeChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->newTemplateFilter((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->mapgenChanged((*reinterpret_cast< MapGenerator(*)>(_a[1]))); break;
        case 5: _t->mazeSizeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->mapFeatureSizeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->drawMapRequested(); break;
        case 8: _t->drawnMapChanged((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 9: _t->askForGeneratedPreview(); break;
        case 10: _t->setSeed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->setScript((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 12: _t->setMap((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->setTheme((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: _t->setTemplateFilter((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->setMapgen((*reinterpret_cast< MapGenerator(*)>(_a[1]))); break;
        case 16: _t->setMazeSize((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->setFeatureSize((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->setDrawnMapData((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 19: _t->setAllMapParameters((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< MapGenerator(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5])),(*reinterpret_cast< int(*)>(_a[6]))); break;
        case 20: _t->updateModelViews(); break;
        case 21: _t->onPreviewMapDestroyed((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 22: _t->setMaster((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 23: _t->setRandomWithoutDLC((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 24: _t->onImageReceived((*reinterpret_cast< const QPixmap(*)>(_a[1]))); break;
        case 25: _t->setHHLimit((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 26: _t->setRandomSeed(); break;
        case 27: _t->setRandomTheme(); break;
        case 28: _t->setRandomMap(); break;
        case 29: _t->setNewSeed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 30: _t->mapTypeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 31: _t->showThemePrompt(); break;
        case 32: _t->updateTheme((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 33: _t->staticMapChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 34: _t->staticMapChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 35: _t->missionMapChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 36: _t->missionMapChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 37: _t->loadDrawing(); break;
        case 38: _t->showSeedPrompt(); break;
        case 39: _t->previewClicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HWMapContainer::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWMapContainer::seedChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (HWMapContainer::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWMapContainer::mapChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (HWMapContainer::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWMapContainer::themeChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (HWMapContainer::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWMapContainer::newTemplateFilter)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (HWMapContainer::*)(MapGenerator );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWMapContainer::mapgenChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (HWMapContainer::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWMapContainer::mazeSizeChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (HWMapContainer::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWMapContainer::mapFeatureSizeChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (HWMapContainer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWMapContainer::drawMapRequested)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (HWMapContainer::*)(const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWMapContainer::drawnMapChanged)) {
                *result = 8;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<HWMapContainer *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->isMaster(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<HWMapContainer *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setMaster(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject HWMapContainer::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_HWMapContainer.data,
    qt_meta_data_HWMapContainer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *HWMapContainer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HWMapContainer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_HWMapContainer.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int HWMapContainer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 40)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 40;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void HWMapContainer::seedChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void HWMapContainer::mapChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void HWMapContainer::themeChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void HWMapContainer::newTemplateFilter(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void HWMapContainer::mapgenChanged(MapGenerator _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void HWMapContainer::mazeSizeChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void HWMapContainer::mapFeatureSizeChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void HWMapContainer::drawMapRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void HWMapContainer::drawnMapChanged(const QByteArray & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
