/****************************************************************************
** Meta object code from reading C++ file 'gamecfgwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../QTfrontend/ui/widget/gamecfgwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'gamecfgwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_GameCFGWidget_t {
    QByteArrayData data[38];
    char stringdata0[465];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_GameCFGWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_GameCFGWidget_t qt_meta_stringdata_GameCFGWidget = {
    {
QT_MOC_LITERAL(0, 0, 13), // "GameCFGWidget"
QT_MOC_LITERAL(1, 14, 12), // "paramChanged"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 5), // "param"
QT_MOC_LITERAL(4, 34, 5), // "value"
QT_MOC_LITERAL(5, 40, 11), // "goToSchemes"
QT_MOC_LITERAL(6, 52, 11), // "goToWeapons"
QT_MOC_LITERAL(7, 64, 11), // "goToDrawMap"
QT_MOC_LITERAL(8, 76, 8), // "setParam"
QT_MOC_LITERAL(9, 85, 13), // "fullNetConfig"
QT_MOC_LITERAL(10, 99, 16), // "resendSchemeData"
QT_MOC_LITERAL(11, 116, 14), // "resendAmmoData"
QT_MOC_LITERAL(12, 131, 17), // "resetSchemeStates"
QT_MOC_LITERAL(13, 149, 9), // "setMaster"
QT_MOC_LITERAL(14, 159, 6), // "master"
QT_MOC_LITERAL(15, 166, 9), // "setTabbed"
QT_MOC_LITERAL(16, 176, 6), // "tabbed"
QT_MOC_LITERAL(17, 183, 11), // "ammoChanged"
QT_MOC_LITERAL(18, 195, 5), // "index"
QT_MOC_LITERAL(19, 201, 10), // "mapChanged"
QT_MOC_LITERAL(20, 212, 21), // "templateFilterChanged"
QT_MOC_LITERAL(21, 234, 11), // "seedChanged"
QT_MOC_LITERAL(22, 246, 12), // "themeChanged"
QT_MOC_LITERAL(23, 259, 13), // "schemeChanged"
QT_MOC_LITERAL(24, 273, 25), // "updateSchemeEnabledStates"
QT_MOC_LITERAL(25, 299, 11), // "scriptIndex"
QT_MOC_LITERAL(26, 311, 13), // "scriptChanged"
QT_MOC_LITERAL(27, 325, 13), // "jumpToSchemes"
QT_MOC_LITERAL(28, 339, 13), // "jumpToWeapons"
QT_MOC_LITERAL(29, 353, 13), // "mapgenChanged"
QT_MOC_LITERAL(30, 367, 12), // "MapGenerator"
QT_MOC_LITERAL(31, 380, 1), // "m"
QT_MOC_LITERAL(32, 382, 16), // "maze_sizeChanged"
QT_MOC_LITERAL(33, 399, 1), // "s"
QT_MOC_LITERAL(34, 401, 23), // "slMapFeatureSizeChanged"
QT_MOC_LITERAL(35, 425, 17), // "onDrawnMapChanged"
QT_MOC_LITERAL(36, 443, 4), // "data"
QT_MOC_LITERAL(37, 448, 16) // "updateModelViews"

    },
    "GameCFGWidget\0paramChanged\0\0param\0"
    "value\0goToSchemes\0goToWeapons\0goToDrawMap\0"
    "setParam\0fullNetConfig\0resendSchemeData\0"
    "resendAmmoData\0resetSchemeStates\0"
    "setMaster\0master\0setTabbed\0tabbed\0"
    "ammoChanged\0index\0mapChanged\0"
    "templateFilterChanged\0seedChanged\0"
    "themeChanged\0schemeChanged\0"
    "updateSchemeEnabledStates\0scriptIndex\0"
    "scriptChanged\0jumpToSchemes\0jumpToWeapons\0"
    "mapgenChanged\0MapGenerator\0m\0"
    "maze_sizeChanged\0s\0slMapFeatureSizeChanged\0"
    "onDrawnMapChanged\0data\0updateModelViews"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_GameCFGWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      26,   14, // methods
       1,  210, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,  144,    2, 0x06 /* Public */,
       5,    1,  149,    2, 0x06 /* Public */,
       6,    1,  152,    2, 0x06 /* Public */,
       7,    0,  155,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    2,  156,    2, 0x0a /* Public */,
       9,    0,  161,    2, 0x0a /* Public */,
      10,    0,  162,    2, 0x0a /* Public */,
      11,    0,  163,    2, 0x0a /* Public */,
      12,    0,  164,    2, 0x0a /* Public */,
      13,    1,  165,    2, 0x0a /* Public */,
      15,    1,  168,    2, 0x0a /* Public */,
      17,    1,  171,    2, 0x08 /* Private */,
      19,    1,  174,    2, 0x08 /* Private */,
      20,    1,  177,    2, 0x08 /* Private */,
      21,    1,  180,    2, 0x08 /* Private */,
      22,    1,  183,    2, 0x08 /* Private */,
      23,    1,  186,    2, 0x08 /* Private */,
      24,    1,  189,    2, 0x08 /* Private */,
      26,    1,  192,    2, 0x08 /* Private */,
      27,    0,  195,    2, 0x08 /* Private */,
      28,    0,  196,    2, 0x08 /* Private */,
      29,    1,  197,    2, 0x08 /* Private */,
      32,    1,  200,    2, 0x08 /* Private */,
      34,    1,  203,    2, 0x08 /* Private */,
      35,    1,  206,    2, 0x08 /* Private */,
      37,    0,  209,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QStringList,    3,    4,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QStringList,    3,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void, QMetaType::Int,   18,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,   25,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 30,   31,
    QMetaType::Void, QMetaType::Int,   33,
    QMetaType::Void, QMetaType::Int,   33,
    QMetaType::Void, QMetaType::QByteArray,   36,
    QMetaType::Void,

 // properties: name, type, flags
      14, QMetaType::Bool, 0x00095103,

       0        // eod
};

void GameCFGWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<GameCFGWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->paramChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2]))); break;
        case 1: _t->goToSchemes((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->goToWeapons((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->goToDrawMap(); break;
        case 4: _t->setParam((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2]))); break;
        case 5: _t->fullNetConfig(); break;
        case 6: _t->resendSchemeData(); break;
        case 7: _t->resendAmmoData(); break;
        case 8: _t->resetSchemeStates(); break;
        case 9: _t->setMaster((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->setTabbed((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->ammoChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->mapChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->templateFilterChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->seedChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: _t->themeChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 16: _t->schemeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->updateSchemeEnabledStates((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->scriptChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->jumpToSchemes(); break;
        case 20: _t->jumpToWeapons(); break;
        case 21: _t->mapgenChanged((*reinterpret_cast< MapGenerator(*)>(_a[1]))); break;
        case 22: _t->maze_sizeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 23: _t->slMapFeatureSizeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 24: _t->onDrawnMapChanged((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 25: _t->updateModelViews(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (GameCFGWidget::*)(const QString & , const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GameCFGWidget::paramChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (GameCFGWidget::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GameCFGWidget::goToSchemes)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (GameCFGWidget::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GameCFGWidget::goToWeapons)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (GameCFGWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GameCFGWidget::goToDrawMap)) {
                *result = 3;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<GameCFGWidget *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->isMaster(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<GameCFGWidget *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setMaster(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject GameCFGWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QGroupBox::staticMetaObject>(),
    qt_meta_stringdata_GameCFGWidget.data,
    qt_meta_data_GameCFGWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *GameCFGWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GameCFGWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_GameCFGWidget.stringdata0))
        return static_cast<void*>(this);
    return QGroupBox::qt_metacast(_clname);
}

int GameCFGWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGroupBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 26;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void GameCFGWidget::paramChanged(const QString & _t1, const QStringList & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void GameCFGWidget::goToSchemes(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void GameCFGWidget::goToWeapons(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void GameCFGWidget::goToDrawMap()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
