/****************************************************************************
** Meta object code from reading C++ file 'teamselhelper.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../QTfrontend/ui/widget/teamselhelper.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'teamselhelper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TeamLabel_t {
    QByteArrayData data[5];
    char stringdata0[53];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TeamLabel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TeamLabel_t qt_meta_stringdata_TeamLabel = {
    {
QT_MOC_LITERAL(0, 0, 9), // "TeamLabel"
QT_MOC_LITERAL(1, 10, 13), // "teamActivated"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 9), // "team_name"
QT_MOC_LITERAL(4, 35, 17) // "teamButtonClicked"

    },
    "TeamLabel\0teamActivated\0\0team_name\0"
    "teamButtonClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TeamLabel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   27,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void TeamLabel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TeamLabel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->teamActivated((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->teamButtonClicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TeamLabel::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TeamLabel::teamActivated)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TeamLabel::staticMetaObject = { {
    QMetaObject::SuperData::link<QLabel::staticMetaObject>(),
    qt_meta_stringdata_TeamLabel.data,
    qt_meta_data_TeamLabel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TeamLabel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TeamLabel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TeamLabel.stringdata0))
        return static_cast<void*>(this);
    return QLabel::qt_metacast(_clname);
}

int TeamLabel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLabel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void TeamLabel::teamActivated(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_TeamShowWidget_t {
    QByteArrayData data[11];
    char stringdata0[133];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TeamShowWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TeamShowWidget_t qt_meta_stringdata_TeamShowWidget = {
    {
QT_MOC_LITERAL(0, 0, 14), // "TeamShowWidget"
QT_MOC_LITERAL(1, 15, 17), // "teamStatusChanged"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 6), // "HWTeam"
QT_MOC_LITERAL(4, 41, 11), // "hhNmChanged"
QT_MOC_LITERAL(5, 53, 16), // "teamColorChanged"
QT_MOC_LITERAL(6, 70, 15), // "changeTeamColor"
QT_MOC_LITERAL(7, 86, 5), // "color"
QT_MOC_LITERAL(8, 92, 12), // "hhNumChanged"
QT_MOC_LITERAL(9, 105, 12), // "activateTeam"
QT_MOC_LITERAL(10, 118, 14) // "onColorChanged"

    },
    "TeamShowWidget\0teamStatusChanged\0\0"
    "HWTeam\0hhNmChanged\0teamColorChanged\0"
    "changeTeamColor\0color\0hhNumChanged\0"
    "activateTeam\0onColorChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TeamShowWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,
       4,    1,   57,    2, 0x06 /* Public */,
       5,    1,   60,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,   63,    2, 0x0a /* Public */,
       6,    0,   66,    2, 0x2a /* Public | MethodCloned */,
       8,    0,   67,    2, 0x0a /* Public */,
       9,    0,   68,    2, 0x08 /* Private */,
      10,    1,   69,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    7,

       0        // eod
};

void TeamShowWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TeamShowWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->teamStatusChanged((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 1: _t->hhNmChanged((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 2: _t->teamColorChanged((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 3: _t->changeTeamColor((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->changeTeamColor(); break;
        case 5: _t->hhNumChanged(); break;
        case 6: _t->activateTeam(); break;
        case 7: _t->onColorChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TeamShowWidget::*)(const HWTeam & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TeamShowWidget::teamStatusChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TeamShowWidget::*)(const HWTeam & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TeamShowWidget::hhNmChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TeamShowWidget::*)(const HWTeam & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TeamShowWidget::teamColorChanged)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TeamShowWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_TeamShowWidget.data,
    qt_meta_data_TeamShowWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TeamShowWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TeamShowWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TeamShowWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TeamShowWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void TeamShowWidget::teamStatusChanged(const HWTeam & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TeamShowWidget::hhNmChanged(const HWTeam & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void TeamShowWidget::teamColorChanged(const HWTeam & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
