/****************************************************************************
** Meta object code from reading C++ file 'teamselect.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../QTfrontend/ui/widget/teamselect.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'teamselect.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TeamSelWidget_t {
    QByteArrayData data[17];
    char stringdata0[233];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TeamSelWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TeamSelWidget_t qt_meta_stringdata_TeamSelWidget = {
    {
QT_MOC_LITERAL(0, 0, 13), // "TeamSelWidget"
QT_MOC_LITERAL(1, 14, 19), // "setEnabledGameStart"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 12), // "teamWillPlay"
QT_MOC_LITERAL(4, 48, 6), // "HWTeam"
QT_MOC_LITERAL(5, 55, 4), // "team"
QT_MOC_LITERAL(6, 60, 14), // "teamNotPlaying"
QT_MOC_LITERAL(7, 75, 15), // "hhogsNumChanged"
QT_MOC_LITERAL(8, 91, 16), // "teamColorChanged"
QT_MOC_LITERAL(9, 108, 15), // "acceptRequested"
QT_MOC_LITERAL(10, 124, 7), // "addTeam"
QT_MOC_LITERAL(11, 132, 11), // "changeHHNum"
QT_MOC_LITERAL(12, 144, 15), // "changeTeamColor"
QT_MOC_LITERAL(13, 160, 16), // "changeTeamStatus"
QT_MOC_LITERAL(14, 177, 20), // "pre_changeTeamStatus"
QT_MOC_LITERAL(15, 198, 12), // "hhNumChanged"
QT_MOC_LITERAL(16, 211, 21) // "proxyTeamColorChanged"

    },
    "TeamSelWidget\0setEnabledGameStart\0\0"
    "teamWillPlay\0HWTeam\0team\0teamNotPlaying\0"
    "hhogsNumChanged\0teamColorChanged\0"
    "acceptRequested\0addTeam\0changeHHNum\0"
    "changeTeamColor\0changeTeamStatus\0"
    "pre_changeTeamStatus\0hhNumChanged\0"
    "proxyTeamColorChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TeamSelWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x06 /* Public */,
       3,    1,   82,    2, 0x06 /* Public */,
       6,    1,   85,    2, 0x06 /* Public */,
       7,    1,   88,    2, 0x06 /* Public */,
       8,    1,   91,    2, 0x06 /* Public */,
       9,    1,   94,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    1,   97,    2, 0x0a /* Public */,
      11,    1,  100,    2, 0x0a /* Public */,
      12,    1,  103,    2, 0x0a /* Public */,
      13,    1,  106,    2, 0x0a /* Public */,
      14,    1,  109,    2, 0x08 /* Private */,
      15,    1,  112,    2, 0x08 /* Private */,
      16,    1,  115,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, 0x80000000 | 4,    5,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 4,    5,

       0        // eod
};

void TeamSelWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TeamSelWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setEnabledGameStart((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->teamWillPlay((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 2: _t->teamNotPlaying((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 3: _t->hhogsNumChanged((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 4: _t->teamColorChanged((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 5: _t->acceptRequested((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 6: _t->addTeam((*reinterpret_cast< HWTeam(*)>(_a[1]))); break;
        case 7: _t->changeHHNum((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 8: _t->changeTeamColor((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 9: _t->changeTeamStatus((*reinterpret_cast< HWTeam(*)>(_a[1]))); break;
        case 10: _t->pre_changeTeamStatus((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 11: _t->hhNumChanged((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 12: _t->proxyTeamColorChanged((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TeamSelWidget::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TeamSelWidget::setEnabledGameStart)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TeamSelWidget::*)(const HWTeam & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TeamSelWidget::teamWillPlay)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TeamSelWidget::*)(const HWTeam & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TeamSelWidget::teamNotPlaying)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TeamSelWidget::*)(const HWTeam & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TeamSelWidget::hhogsNumChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (TeamSelWidget::*)(const HWTeam & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TeamSelWidget::teamColorChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (TeamSelWidget::*)(const HWTeam & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TeamSelWidget::acceptRequested)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TeamSelWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QGroupBox::staticMetaObject>(),
    qt_meta_stringdata_TeamSelWidget.data,
    qt_meta_data_TeamSelWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TeamSelWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TeamSelWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TeamSelWidget.stringdata0))
        return static_cast<void*>(this);
    return QGroupBox::qt_metacast(_clname);
}

int TeamSelWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGroupBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void TeamSelWidget::setEnabledGameStart(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TeamSelWidget::teamWillPlay(const HWTeam & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void TeamSelWidget::teamNotPlaying(const HWTeam & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void TeamSelWidget::hhogsNumChanged(const HWTeam & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void TeamSelWidget::teamColorChanged(const HWTeam & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void TeamSelWidget::acceptRequested(const HWTeam & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
