/****************************************************************************
** Meta object code from reading C++ file 'drawmapscene.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../QTfrontend/drawmapscene.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'drawmapscene.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DrawMapScene_t {
    QByteArrayData data[17];
    char stringdata0[173];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DrawMapScene_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DrawMapScene_t qt_meta_stringdata_DrawMapScene = {
    {
QT_MOC_LITERAL(0, 0, 12), // "DrawMapScene"
QT_MOC_LITERAL(1, 13, 11), // "pathChanged"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 16), // "brushSizeChanged"
QT_MOC_LITERAL(4, 43, 9), // "brushSize"
QT_MOC_LITERAL(5, 53, 4), // "undo"
QT_MOC_LITERAL(6, 58, 8), // "clearMap"
QT_MOC_LITERAL(7, 67, 12), // "simplifyLast"
QT_MOC_LITERAL(8, 80, 8), // "optimize"
QT_MOC_LITERAL(9, 89, 10), // "setErasing"
QT_MOC_LITERAL(10, 100, 7), // "erasing"
QT_MOC_LITERAL(11, 108, 10), // "showCursor"
QT_MOC_LITERAL(12, 119, 10), // "hideCursor"
QT_MOC_LITERAL(13, 130, 11), // "setPathType"
QT_MOC_LITERAL(14, 142, 8), // "PathType"
QT_MOC_LITERAL(15, 151, 8), // "pathType"
QT_MOC_LITERAL(16, 160, 12) // "setBrushSize"

    },
    "DrawMapScene\0pathChanged\0\0brushSizeChanged\0"
    "brushSize\0undo\0clearMap\0simplifyLast\0"
    "optimize\0setErasing\0erasing\0showCursor\0"
    "hideCursor\0setPathType\0PathType\0"
    "pathType\0setBrushSize"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DrawMapScene[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   69,    2, 0x06 /* Public */,
       3,    1,   70,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   73,    2, 0x0a /* Public */,
       6,    0,   74,    2, 0x0a /* Public */,
       7,    0,   75,    2, 0x0a /* Public */,
       8,    0,   76,    2, 0x0a /* Public */,
       9,    1,   77,    2, 0x0a /* Public */,
      11,    0,   80,    2, 0x0a /* Public */,
      12,    0,   81,    2, 0x0a /* Public */,
      13,    1,   82,    2, 0x0a /* Public */,
      16,    1,   85,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void, QMetaType::Int,    4,

       0        // eod
};

void DrawMapScene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DrawMapScene *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->pathChanged(); break;
        case 1: _t->brushSizeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->undo(); break;
        case 3: _t->clearMap(); break;
        case 4: _t->simplifyLast(); break;
        case 5: _t->optimize(); break;
        case 6: _t->setErasing((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->showCursor(); break;
        case 8: _t->hideCursor(); break;
        case 9: _t->setPathType((*reinterpret_cast< PathType(*)>(_a[1]))); break;
        case 10: _t->setBrushSize((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DrawMapScene::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DrawMapScene::pathChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DrawMapScene::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DrawMapScene::brushSizeChanged)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DrawMapScene::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsScene::staticMetaObject>(),
    qt_meta_stringdata_DrawMapScene.data,
    qt_meta_data_DrawMapScene,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DrawMapScene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DrawMapScene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DrawMapScene.stringdata0))
        return static_cast<void*>(this);
    return QGraphicsScene::qt_metacast(_clname);
}

int DrawMapScene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsScene::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void DrawMapScene::pathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DrawMapScene::brushSizeChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
