/****************************************************************************
** Meta object code from reading C++ file 'selectWeapon.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../QTfrontend/ui/widget/selectWeapon.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'selectWeapon.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SelWeaponItem_t {
    QByteArrayData data[1];
    char stringdata0[14];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SelWeaponItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SelWeaponItem_t qt_meta_stringdata_SelWeaponItem = {
    {
QT_MOC_LITERAL(0, 0, 13) // "SelWeaponItem"

    },
    "SelWeaponItem"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SelWeaponItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void SelWeaponItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SelWeaponItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_SelWeaponItem.data,
    qt_meta_data_SelWeaponItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SelWeaponItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SelWeaponItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SelWeaponItem.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int SelWeaponItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_SelWeaponWidget_t {
    QByteArrayData data[17];
    char stringdata0[190];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SelWeaponWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SelWeaponWidget_t qt_meta_stringdata_SelWeaponWidget = {
    {
QT_MOC_LITERAL(0, 0, 15), // "SelWeaponWidget"
QT_MOC_LITERAL(1, 16, 14), // "weaponsDeleted"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 11), // "weaponsName"
QT_MOC_LITERAL(4, 44, 12), // "weaponsAdded"
QT_MOC_LITERAL(5, 57, 4), // "ammo"
QT_MOC_LITERAL(6, 62, 13), // "weaponsEdited"
QT_MOC_LITERAL(7, 76, 14), // "oldWeaponsName"
QT_MOC_LITERAL(8, 91, 14), // "newWeaponsName"
QT_MOC_LITERAL(9, 106, 10), // "setDefault"
QT_MOC_LITERAL(10, 117, 10), // "setWeapons"
QT_MOC_LITERAL(11, 128, 14), // "setWeaponsName"
QT_MOC_LITERAL(12, 143, 4), // "name"
QT_MOC_LITERAL(13, 148, 13), // "switchWeapons"
QT_MOC_LITERAL(14, 162, 17), // "deleteWeaponsName"
QT_MOC_LITERAL(15, 180, 4), // "save"
QT_MOC_LITERAL(16, 185, 4) // "copy"

    },
    "SelWeaponWidget\0weaponsDeleted\0\0"
    "weaponsName\0weaponsAdded\0ammo\0"
    "weaponsEdited\0oldWeaponsName\0"
    "newWeaponsName\0setDefault\0setWeapons\0"
    "setWeaponsName\0name\0switchWeapons\0"
    "deleteWeaponsName\0save\0copy"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SelWeaponWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x06 /* Public */,
       4,    2,   72,    2, 0x06 /* Public */,
       6,    3,   77,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,   84,    2, 0x0a /* Public */,
      10,    1,   85,    2, 0x0a /* Public */,
      11,    1,   88,    2, 0x0a /* Public */,
      13,    1,   91,    2, 0x0a /* Public */,
      14,    0,   94,    2, 0x0a /* Public */,
       8,    0,   95,    2, 0x0a /* Public */,
      15,    0,   96,    2, 0x0a /* Public */,
      16,    0,   97,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,    5,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString,    7,    8,    5,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SelWeaponWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SelWeaponWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->weaponsDeleted((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->weaponsAdded((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 2: _t->weaponsEdited((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2])),(*reinterpret_cast< QString(*)>(_a[3]))); break;
        case 3: _t->setDefault(); break;
        case 4: _t->setWeapons((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->setWeaponsName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->switchWeapons((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->deleteWeaponsName(); break;
        case 8: _t->newWeaponsName(); break;
        case 9: _t->save(); break;
        case 10: _t->copy(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SelWeaponWidget::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SelWeaponWidget::weaponsDeleted)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SelWeaponWidget::*)(QString , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SelWeaponWidget::weaponsAdded)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (SelWeaponWidget::*)(QString , QString , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SelWeaponWidget::weaponsEdited)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SelWeaponWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_SelWeaponWidget.data,
    qt_meta_data_SelWeaponWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SelWeaponWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SelWeaponWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SelWeaponWidget.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int SelWeaponWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void SelWeaponWidget::weaponsDeleted(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void SelWeaponWidget::weaponsAdded(QString _t1, QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void SelWeaponWidget::weaponsEdited(QString _t1, QString _t2, QString _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
