/****************************************************************************
** Meta object code from reading C++ file 'drawmapwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../QTfrontend/ui/widget/drawmapwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'drawmapwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DrawMapView_t {
    QByteArrayData data[1];
    char stringdata0[12];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DrawMapView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DrawMapView_t qt_meta_stringdata_DrawMapView = {
    {
QT_MOC_LITERAL(0, 0, 11) // "DrawMapView"

    },
    "DrawMapView"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DrawMapView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void DrawMapView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject DrawMapView::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsView::staticMetaObject>(),
    qt_meta_stringdata_DrawMapView.data,
    qt_meta_data_DrawMapView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DrawMapView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DrawMapView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DrawMapView.stringdata0))
        return static_cast<void*>(this);
    return QGraphicsView::qt_metacast(_clname);
}

int DrawMapView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsView::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_DrawMapWidget_t {
    QByteArrayData data[18];
    char stringdata0[191];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DrawMapWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DrawMapWidget_t qt_meta_stringdata_DrawMapWidget = {
    {
QT_MOC_LITERAL(0, 0, 13), // "DrawMapWidget"
QT_MOC_LITERAL(1, 14, 16), // "brushSizeChanged"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 9), // "brushSize"
QT_MOC_LITERAL(4, 42, 4), // "undo"
QT_MOC_LITERAL(5, 47, 5), // "clear"
QT_MOC_LITERAL(6, 53, 8), // "optimize"
QT_MOC_LITERAL(7, 62, 10), // "setErasing"
QT_MOC_LITERAL(8, 73, 7), // "erasing"
QT_MOC_LITERAL(9, 81, 4), // "save"
QT_MOC_LITERAL(10, 86, 8), // "fileName"
QT_MOC_LITERAL(11, 95, 4), // "load"
QT_MOC_LITERAL(12, 100, 11), // "setPathType"
QT_MOC_LITERAL(13, 112, 22), // "DrawMapScene::PathType"
QT_MOC_LITERAL(14, 135, 8), // "pathType"
QT_MOC_LITERAL(15, 144, 12), // "setBrushSize"
QT_MOC_LITERAL(16, 157, 11), // "pathChanged"
QT_MOC_LITERAL(17, 169, 21) // "brushSizeChanged_slot"

    },
    "DrawMapWidget\0brushSizeChanged\0\0"
    "brushSize\0undo\0clear\0optimize\0setErasing\0"
    "erasing\0save\0fileName\0load\0setPathType\0"
    "DrawMapScene::PathType\0pathType\0"
    "setBrushSize\0pathChanged\0brushSizeChanged_slot"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DrawMapWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   72,    2, 0x0a /* Public */,
       5,    0,   73,    2, 0x0a /* Public */,
       6,    0,   74,    2, 0x0a /* Public */,
       7,    1,   75,    2, 0x0a /* Public */,
       9,    1,   78,    2, 0x0a /* Public */,
      11,    1,   81,    2, 0x0a /* Public */,
      12,    1,   84,    2, 0x0a /* Public */,
      15,    1,   87,    2, 0x0a /* Public */,
      16,    0,   90,    2, 0x08 /* Private */,
      17,    1,   91,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    3,

       0        // eod
};

void DrawMapWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DrawMapWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->brushSizeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->undo(); break;
        case 2: _t->clear(); break;
        case 3: _t->optimize(); break;
        case 4: _t->setErasing((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->save((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->load((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->setPathType((*reinterpret_cast< DrawMapScene::PathType(*)>(_a[1]))); break;
        case 8: _t->setBrushSize((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->pathChanged(); break;
        case 10: _t->brushSizeChanged_slot((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DrawMapWidget::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DrawMapWidget::brushSizeChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DrawMapWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_DrawMapWidget.data,
    qt_meta_data_DrawMapWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DrawMapWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DrawMapWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DrawMapWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DrawMapWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void DrawMapWidget::brushSizeChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
